#!/usr/bin/env python3
import time
import fcntl
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions
#from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects
from qualys_etl.etld_knowledgebase import knowledgebase_02_workflow_manager


def begin_knowledgebase_03_extract_controller():
    etld_lib_functions.logger.info(f"start")


def end_knowledgebase_03_extract_controller():
    etld_lib_functions.logger.info(f"end")


def knowledgebase_extract_controller(kb_last_modified_after=None, lock_file_required=False):
    if kb_last_modified_after is not None:
        etld_lib_config.kb_last_modified_after = kb_last_modified_after
    else:
        etld_lib_config.kb_last_modified_after = ""

    def get_knowledgebase_data(distribute_data=False):
        try:
            # knowledgebase_04_extract_wrapper(message=f"kb_last_modified_after={etld_lib_config.kb_last_modified_after}")
            knowledgebase_02_workflow_manager.knowledgebase_04_extract_wrapper(
                message=f"kb_last_modified_after={etld_lib_config.kb_last_modified_after}")
            if distribute_data:
                knowledgebase_02_workflow_manager.knowledgebase_05_transform_load_xml_to_sqlite_wrapper()
                knowledgebase_02_workflow_manager.knowledgebase_06_distribution_wrapper()
        except Exception as e2:
            etld_lib_functions.logger.warning(f"Error get_knowledgebase_data")
            etld_lib_functions.logger.warning(f"Exception: {e2}")
            raise Exception("Raise Exception from get_knowledgebase_data in knowledgebase_extract_controller")

    retry_attempts = 12
    retry_attempt = 0
    retry_sleep_time = 300
    while retry_attempt < retry_attempts:
        retry_attempt += 1
        try:
            if lock_file_required:
                with open(etld_lib_config.kb_lock_file, 'wb+') as lock_knowledgebase_fcntl:
                    fcntl.flock(lock_knowledgebase_fcntl, fcntl.LOCK_EX | fcntl.LOCK_NB)
                    get_knowledgebase_data(distribute_data=True) # targeted run from other module.
                    break # Success
            else:
                get_knowledgebase_data(distribute_data=False) # Normal Run through spawn workflow
                break # Success
        except Exception as e:
            etld_lib_functions.logger.warning(f"Exception from knowledgebase_extract_controller.  {__file__} ")
            etld_lib_functions.logger.warning(f"Exception: {e}")
            etld_lib_functions.logger.warning(f"Retrying in {retry_sleep_time} seconds")
            time.sleep(retry_sleep_time)
    else:
        etld_lib_functions.logger.error(f"Retried {retry_attempt} times with no success.")
        etld_lib_functions.logger.error(f"Check to see if KnowledgeBase Program is already running.  {__file__} ")
        exit(1)


def main(kb_last_modified_after=None, lock_file_required=False):
    if kb_last_modified_after is None:
        kb_last_modified_after = etld_lib_config.kb_last_modified_after
    begin_knowledgebase_03_extract_controller()
    knowledgebase_extract_controller(kb_last_modified_after=kb_last_modified_after, lock_file_required=lock_file_required)
    end_knowledgebase_03_extract_controller()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='knowledgebase_extract_03_controller')
    etld_lib_config.main()
    #etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main(lock_file_required=True)


#!/usr/bin/env python3
import sys
import timeit
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_config
#from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects

from qualys_etl.etld_asset_inventory import asset_inventory_03_extract_controller
from qualys_etl.etld_asset_inventory import asset_inventory_05_transform_load_json_to_sqlite
from qualys_etl.etld_asset_inventory import asset_inventory_06_distribution
import traceback
import time

global start_time
global stop_time


def asset_inventory_03_extract_controller_wrapper(
        module_function=asset_inventory_03_extract_controller, message=""
):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def asset_inventory_05_transform_load_json_to_sqlite_wrapper(
        module_function=asset_inventory_05_transform_load_json_to_sqlite, message=""
):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def asset_inventory_06_distribution_wrapper(
        module_function=asset_inventory_06_distribution, message=""):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def begin_asset_inventory_02_workflow_manager():
    global start_time
    start_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"__start__ asset_inventory_etl_workflow {str(sys.argv)}")


def end_asset_inventory_workflow_manager():
    global start_time
    global stop_time

    stop_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"runtime for asset_inventory_etl_workflow in seconds: {stop_time - start_time:,}")
    etld_lib_functions.logger.info(f"__end__ asset_inventory_etl_workflow {str(sys.argv)}")


def asset_inventory_etl_workflow():
    try:
        begin_asset_inventory_02_workflow_manager()
        asset_inventory_03_extract_controller_wrapper(
            message=f"asset_last_updated={etld_lib_config.asset_inventory_asset_last_updated}")
        asset_inventory_06_distribution_wrapper()
        end_asset_inventory_workflow_manager()
    except Exception as e:
        time.sleep(10)
        etld_lib_functions.logger.error(f"Error asset_inventory_02_workflow_manager - asset_inventory_etl_workflow, "
                                        f"please investigate {sys.argv}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)
    # except KeyboardInterrupt:
    #     time.sleep(10)
    #     etld_lib_functions.logger.error(f"Error asset_inventory_02_workflow_manager - asset_inventory_etl_workflow, "
    #                                     f"please investigate {sys.argv}")
    #     etld_lib_functions.logger.error("Caught a KeyboardInterrupt, performing cleanup...")
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)
    # except SystemExit:
    #     time.sleep(10)
    #     etld_lib_functions.logger.error(f"Error asset_inventory_02_workflow_manager - asset_inventory_etl_workflow, "
    #                                     f"please investigate {sys.argv}")
    #     etld_lib_functions.logger.error("Caught a SystemExit, performing cleanup...")
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)

def main():
    asset_inventory_etl_workflow()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='asset_inventory_02_workflow_manager')
    etld_lib_config.main()
    #etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main()



#!/usr/bin/env python3
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_spawn_etl
from qualys_etl.etld_pcrs import pcrs_02_workflow_manager


# def main():
#     etld_lib_config.set_path_qetl_user_home_dir()
#     etld_lib_spawn_etl.log_dir = etld_lib_config.qetl_user_log_dir
#     etld_lib_spawn_etl.log_file_path = etld_lib_config.pcrs_log_file
#     etld_lib_spawn_etl.log_file_rotate_path = etld_lib_config.pcrs_log_rotate_file
#     etld_lib_spawn_etl.lock_file = etld_lib_config.pcrs_lock_file
#     etld_lib_spawn_etl.log_file_max_size = etld_lib_config.pcrs_log_file_max_size                                            # 1024 * size = Max Meg Size
#     etld_lib_spawn_etl.spawned_process_max_run_time = etld_lib_config.pcrs_spawned_process_max_run_time                      # seconds before terminate as there is an issue.
#     etld_lib_spawn_etl.spawned_process_sleep_time = etld_lib_config.pcrs_spawned_process_sleep_time                          # check every n seconds for spawned_process.is_alive()
#     etld_lib_spawn_etl.spawned_process_count_to_status_update = etld_lib_config.pcrs_spawned_process_count_to_status_update  # print status spawned_process.is_alive() after n checks
#     etld_lib_spawn_etl.etl_files_max_time_between_changes = etld_lib_config.pcrs_etl_files_max_time_between_changes
#
#
#     etld_lib_spawn_etl.etl_dir_to_monitor = etld_lib_config.qetl_user_data_dir
#
#     etld_lib_spawn_etl.target_module_to_run = pcrs_02_workflow_manager.pcrs_etl_workflow
#     etld_lib_spawn_etl.target_method_in_module_to_run = "pcrs_etl_workflow"
#     etld_lib_spawn_etl.etl_main()

def main():
    prefix_name = "pcrs"
    etld_lib_spawn_etl.etl_main(etl_workflow_prefix=prefix_name)

if __name__ == '__main__':
    main()



#!/usr/bin/env python3
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_spawn_etl
from qualys_etl.etld_test_system import test_system_02_workflow_manager


# def main():
#     etld_lib_config.set_path_qetl_user_home_dir()
#     etld_lib_spawn_etl.log_dir = etld_lib_config.qetl_user_log_dir
#     etld_lib_spawn_etl.log_file_path = etld_lib_config.test_system_log_file
#     etld_lib_spawn_etl.log_file_rotate_path = etld_lib_config.test_system_log_rotate_file
#     etld_lib_spawn_etl.lock_file = etld_lib_config.test_system_lock_file
#     etld_lib_spawn_etl.log_file_max_size = (1024 * 100000)          # 1024 * size = Max Meg Size
#     etld_lib_spawn_etl.spawned_process_max_run_time = 604800        # seconds before terminate as there is an issue.
#     etld_lib_spawn_etl.spawned_process_sleep_time = 5               # check every n seconds for spawned_process.is_alive()
#     etld_lib_spawn_etl.spawned_process_count_to_status_update = 11  # print status spawned_process.is_alive() after n checks
#
#     etld_lib_spawn_etl.etl_dir_to_monitor = etld_lib_config.qetl_user_data_dir
#     etld_lib_spawn_etl.etl_files_max_time_between_changes = 1800
#
#     etld_lib_spawn_etl.target_module_to_run = test_system_02_workflow_manager.test_system_etl_workflow
#     etld_lib_spawn_etl.target_method_in_module_to_run = "test_system_etl_workflow"
#     etld_lib_spawn_etl.etl_main()

def main():
    prefix_name = "test_system"
    etld_lib_spawn_etl.etl_main(etl_workflow_prefix=prefix_name)


if __name__ == '__main__':
    # main()
    print("Test cases being rewritten, test individual modules until future notice.")
    exit(1)



#!/usr/bin/env python3
from pathlib import Path
import json
import gzip
import re
from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_extract_transform_load as etld_lib_extract_transform_load


def was_webapp_extract(
        batch_number_str, qualys_headers_dict, cred_dict, file_info_dict, last_id=0, page_size=25):

    start_msg_was_extract(function_name='was_webapp_extract')

    payload = '{"ServiceRequest": {' \
              '"preferences": ' \
              '{' \
              '"limitResults": '  f'"{page_size}"'  ', ' \
              '"verbose": "true"' \
              '}, ' \
              '"filters": {' '"Criteria": [' \
              '{"field": "id", "operator": "GREATER", "value": '  f'"{last_id}"' + '},' \
              '{"field": "lastScan.date", "operator": "GREATER", "value": '  \
              f'"{etld_lib_config.was_webapp_last_scan_date}"' + '}' \
              ']}}}'
    if str(etld_lib_config.was_webapp_last_scan_date).startswith("2000"):
        # Get all webapps regardless of scan status.
        payload = '{"ServiceRequest": {' \
                  '"preferences": ' \
                  '{' \
                  '"limitResults": '  f'"{page_size}"'  ', ' \
                  '"verbose": "true"' \
                  '}, ' \
                  '"filters": {' '"Criteria": [' \
                  '{"field": "id", "operator": "GREATER", "value": '  f'"{last_id}"' + '}' \
                  ']}}}'


    # url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/3.0/search/was/webapp"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.was_webapp_search_api_endpoint}"
    headers = {'X-Requested-With': 'qualysetl',
               'Authorization': cred_dict['authorization'],
               'Content-Type': 'application/json',
               'Accept': 'application/json',
               }

    etld_lib_functions.logger.info(f"api call     - URL:{url} - PAYLOAD:{payload}")
    json_file = Path(file_info_dict['next_file_path'])

    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.was_try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=etld_lib_config.was_http_conn_timeout,
        chunk_size_calc=etld_lib_config.was_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json',
        compression_method=etld_lib_config.was_open_file_compression_method)
    end_msg_was_extract(function_name='was_webapp_extract')

    was_webapp_detail_extract_controller(
        batch_number_str, qualys_headers_dict, cred_dict, file_info_dict, last_id=0, page_size=25)


def was_webapp_detail_extract_controller(
        batch_number_str, qualys_headers_dict, cred_dict, file_info_dict, last_id=0, page_size=25):
    # was_webapp_utc_run_datetime_2022-11-01T21:50:59Z_utc_last_scan_date_2022-10-25T00:00:00Z_batch_000001.json.gz
    # was_webapp_detail_utc_run_datetime_2022-11-01T21:50:59Z_utc_last_scan_date_2022-10-25T00:00:00Z_batch_000001_id_[idnum].json.gz

    start_msg_was_extract(function_name=f'was_webapp_detail_extract_{batch_number_str}')
    json_file = Path(file_info_dict['next_file_path'])
    webapp_id_list = []
    with etld_lib_config.was_open_file_compression_method(str(json_file), "rt", encoding='utf-8')  as read_file:
        json_data = json.load(read_file)
        if 'data' in  json_data['ServiceResponse']:
            for data_list_item in json_data['ServiceResponse']['data']:
                if 'WebApp' in data_list_item:
                    if 'id' in data_list_item['WebApp']:
                        webapp_id_list.append(data_list_item['WebApp']['id'])
    if len(webapp_id_list) == 0:
        etld_lib_functions.logger.info(f"No webapps scanned since {etld_lib_config.was_webapp_last_scan_date}")

    for webapp_id in webapp_id_list:

        json_file = Path(file_info_dict['next_file_path'])
        batch_number_str_with_webapp_id = f"{batch_number_str}_webapp_id_{webapp_id}"
        # was_webapp_utc_run_datetime_2022-11-01T21:50:59Z_utc_last_scan_date_2022-10-25T00:00:00Z_batch_000001.json.gz
        # was_webapp_detail_utc_run_datetime_2022-11-01T21:50:59Z_utc_last_scan_date_2022-10-25T00:00:00Z_batch_000001_webapp_id_[idnum].json.gz
        # was_finding_detail_utc_run_datetime_2022-11-01T21:50:59Z_utc_last_scan_date_2022-10-25T00:00:00Z_batch_000001_webapp_id_[idnum].json.gz
        file_info_dict['webapp_id_detail_json_file'] = str(json_file).\
            replace("/was_webapp_utc", "/was_webapp_detail_utc").\
            replace(f"_{batch_number_str}", f"_{batch_number_str_with_webapp_id}")
        file_info_dict['finding_detail_for_webapp_id_json_file'] = str(json_file). \
            replace("/was_webapp_utc", "/was_finding_detail_utc"). \
            replace(f"_{batch_number_str}", f"_{batch_number_str_with_webapp_id}")

        was_webapp_detail_extract_data_for_webapp_id(
            batch_number_str_with_webapp_id, qualys_headers_dict, cred_dict, file_info_dict, webapp_id=webapp_id)

        was_finding_detail_extract_data_for_webapp_id(
            batch_number_str_with_webapp_id, qualys_headers_dict, cred_dict, file_info_dict, webapp_id=webapp_id)

    end_msg_was_extract(function_name=f'was_webapp_detail_extract_batch_{batch_number_str}')


def was_webapp_detail_extract_data_for_webapp_id(
        batch_number_str, qualys_headers_dict, cred_dict, file_info_dict, webapp_id=0):

    json_file = file_info_dict['webapp_id_detail_json_file']

    payload = {}
    #url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/3.0/get/was/webapp/{webapp_id}"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.was_webapp_get_api_endpoint}{webapp_id}"
    headers = {'X-Requested-With': 'qualysetl',
               'Authorization': cred_dict['authorization'],
               'Content-Type': 'application/json',
               'Accept': 'application/json',
               }

    start_msg_was_extract(function_name=f'was_webapp_detail_extract_{batch_number_str}')
    etld_lib_functions.logger.info(f"api call     - URL:{url} - PAYLOAD:{payload}")

    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.was_try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=etld_lib_config.was_http_conn_timeout,
        chunk_size_calc=etld_lib_config.was_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json',
        compression_method=etld_lib_config.was_open_file_compression_method,
        request_method='GET')
    end_msg_was_extract(function_name=f'was_webapp_detail_extract_{batch_number_str}')


def was_finding_detail_extract_data_for_webapp_id(
        batch_number_str, qualys_headers_dict, cred_dict, file_info_dict, webapp_id=0, page_size=100):

    payload_template = '''{
        "ServiceRequest": {
            "preferences": {
                "limitResults": "my_limitResults",
                "verbose": "true"
            },
            "filters": {
                "Criteria": [
                    {
                        "field": "webApp.id",
                        "operator": "EQUALS",
                        "value": "my_webApp.id"
                    },
                    {
                        "field": "id",
                        "operator": "GREATER",
                        "value": "my_lastId"
                    }
                ]
            }
        }}'''

    my_lastId = 0
    json_file_page_number = 0
    payload_template = re.sub('my_webApp.id', str(webapp_id), payload_template, 1)
    payload_template = re.sub('my_limitResults', str(page_size), payload_template, 1)
    payload_template = re.sub('\s+', ' ', payload_template)
    batch_number_str_template = batch_number_str
    json_file_template = file_info_dict['finding_detail_for_webapp_id_json_file']

    break_while_loop_flag = False
    while True:
        json_file_page_number = json_file_page_number + 1
        json_file_page_number_fmt = str(json_file_page_number).zfill(3)
        json_file = re.sub('.json', f"_{json_file_page_number_fmt}.json", json_file_template)
        batch_number_str = batch_number_str_template + f"_{json_file_page_number_fmt}"
        payload = re.sub('my_lastId', str(my_lastId), payload_template, 1)

        # payload = '{"ServiceRequest": {' \
        #           '"preferences": {"limitResults": ' \
        #           f'"{page_size}"' \
        #           ', "verbose": "true"}, ' \
        #           '"filters": {' '"Criteria": ' \
        #           '[{"field": "webApp.id", "operator": "EQUALS", "value": ' \
        #           f'"{webapp_id}"' + '}]}}}'

        start_msg_was_extract(function_name=f'was_finding_detail_extract_{batch_number_str}')
        #url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/3.0/search/was/finding"
        url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.was_finding_search_api_endpoint}"
        headers = {'X-Requested-With': 'qualysetl',
                   'Authorization': cred_dict['authorization'],
                   'Content-Type': 'application/json',
                   'Accept': 'application/json',
                   }

        etld_lib_functions.logger.info(f"api call     - URL:{url} - PAYLOAD:{payload}")

        etld_lib_extract_transform_load.extract_qualys(
            try_extract_max_count=etld_lib_config.was_try_extract_max_count,
            url=url,
            headers=headers,
            payload=payload,
            http_conn_timeout=etld_lib_config.was_http_conn_timeout,
            chunk_size_calc=etld_lib_config.was_chunk_size_calc,
            output_file=json_file,
            cred_dict=cred_dict,
            qualys_headers_multiprocessing_dict=qualys_headers_dict,
            batch_number_formatted=batch_number_str,
            extract_validation_type='json',
            compression_method=etld_lib_config.was_open_file_compression_method)

        with etld_lib_config.was_open_file_compression_method(str(json_file), "rt", encoding='utf-8')  as read_file:
            json_data = json.load(read_file)
            if 'ServiceResponse' in json_data and \
                    'hasMoreRecords' in json_data['ServiceResponse'] and \
                    str(json_data['ServiceResponse']['hasMoreRecords']).lower() == 'true' and \
                    'lastId' in json_data['ServiceResponse']:
                my_lastId = json_data['ServiceResponse']['lastId']
                end_msg_was_extract(function_name=f'was_finding_detail_extract_{batch_number_str}')
            else:
                break_while_loop_flag = True
                end_msg_was_extract(function_name=f'was_finding_detail_extract_{batch_number_str}')

        if break_while_loop_flag:
            break


def was_catalog_extract(
        batch_number_str, qualys_headers_dict, cred_dict, file_info_dict, last_id=0, page_size=25):

    start_msg_was_extract(function_name='was_catalog_extract')
    filter_field = 'id'
    filter_operator = 'GREATER'
    filter_value = last_id
    payload = '{"ServiceRequest": {' \
              '"preferences": {"limitResults": ' f'"{page_size}"'  ', "verbose": "true"}, ' \
              '"filters": {' '"Criteria":  ' \
              '{"field": ' f'"{filter_field}"' ', "operator": ' f'"{filter_operator}"' ', "value": ' f'"{filter_value}"' '}' \
              '}' \
              '}}'

    # url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/3.0/search/was/catalog"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.was_catalog_search_api_endpoint}"
    headers = {'X-Requested-With': 'qualysetl',
               'Authorization': cred_dict['authorization'],
               'Content-Type': 'application/json',
               'Accept': 'application/json',
               }

    etld_lib_functions.logger.info(f"api call     - URL:{url} - PAYLOAD:{payload}")
    json_file = Path(file_info_dict['next_file_path'])

    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.was_try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=etld_lib_config.was_http_conn_timeout,
        chunk_size_calc=etld_lib_config.was_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json',
        compression_method=etld_lib_config.was_open_file_compression_method)
    end_msg_was_extract(function_name='was_catalog_extract')


def was_webapp_extract_count(batch_number_str, qualys_headers_dict, cred_dict, file_info_dict):

    # url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/3.0/count/was/webapp"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.was_webapp_count_api_endpoint}"

    headers = {'X-Requested-With': 'qualysetl',
               'Authorization': cred_dict['authorization'],
               'Content-Type': 'application/json',
               'Accept': 'application/json',
               }

    if str(etld_lib_config.was_webapp_last_scan_date).startswith("2000"):
        # Get all webapps regardless of scan status.
        payload = '{"ServiceRequest": {' \
                  '"filters": {' '"Criteria": [' \
                  '{"field": "id", "operator": "GREATER", "value": '  f'"0"' + '}' \
                  ']}}}'

    else:
        payload = '{"ServiceRequest": {' \
                  '"filters": {' '"Criteria": [' \
                  '{"field": "id", "operator": "GREATER", "value": '  f'"0"' + '},' \
                  '{"field": "lastScan.date", "operator": "GREATER", "value": ' \
                   f'"{etld_lib_config.was_webapp_last_scan_date}"' + '}' \
                   ']}}}'

    etld_lib_functions.logger.info(f"api call     - {url}")
    json_file = Path(file_info_dict['next_file_path'])

    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.was_try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=etld_lib_config.was_http_conn_timeout,
        chunk_size_calc=etld_lib_config.was_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json')

    was_log_count(json_file=json_file, count_type='was_webapp')


def was_finding_extract_count(batch_number_str, qualys_headers_dict, cred_dict, file_info_dict):

    # url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/3.0/count/was/finding"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.was_finding_count_api_endpoint}"

    headers = {'X-Requested-With': 'qualysetl',
               'Authorization': cred_dict['authorization'],
               'Content-Type': 'application/json',
               'Accept': 'application/json',
               }

    etld_lib_functions.logger.info(f"api call     - {url}")

    json_file = Path(file_info_dict['next_file_path'])

    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.was_try_extract_max_count,
        url=url,
        headers=headers,
        payload={},
        http_conn_timeout=etld_lib_config.was_http_conn_timeout,
        chunk_size_calc=etld_lib_config.was_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json')

    was_log_count(json_file=json_file, count_type='was_finding')


def was_catalog_extract_count(batch_number_str, qualys_headers_dict, cred_dict, file_info_dict):

    # url = f"https://{cred_dict['api_fqdn_server']}/qps/rest/3.0/count/was/catalog"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.was_catalog_count_api_endpoint}"

    headers = {'X-Requested-With': 'qualysetl',
               'Authorization': cred_dict['authorization'],
               'Content-Type': 'application/json',
               'Accept': 'application/json',
               }

    payload = '{"ServiceRequest": {' \
              '"filters": {' '"Criteria": [' \
              '{"field": "id", "operator": ' \
              '"GREATER", "value": '  f'"{etld_lib_config.was_catalog_start_greater_than_last_id}"' + '}' \
                  ']}}}'

    filter_field = 'id'
    filter_operator = 'GREATER'
    filter_value = etld_lib_config.was_catalog_start_greater_than_last_id
    payload = '{"ServiceRequest": {' \
              '"filters": {' '"Criteria":  ' \
              '{"field": ' f'"{filter_field}"' ', "operator": ' f'"{filter_operator}"' ', "value": ' f'"{filter_value}"' '}' \
              '}' \
              '}}'

    etld_lib_functions.logger.info(f"api call     - URL:{url} - PAYLOAD:{payload}")
    json_file = Path(file_info_dict['next_file_path'])

    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.was_try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=etld_lib_config.was_http_conn_timeout,
        chunk_size_calc=etld_lib_config.was_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json')

    was_log_count(json_file=json_file, count_type='was_catalog')


def was_log_count(json_file, count_type='was_webapp'):
    try:
        # {"ServiceResponse":{"count":139,"responseCode":"SUCCESS"}}
        with etld_lib_config.was_open_file_compression_method(str(json_file), "rt", encoding='utf-8') as read_file:
            my_count_service_response = json.load(read_file)
            if 'ServiceResponse' in my_count_service_response.keys():
                my_count = my_count_service_response['ServiceResponse']
                if "responseCode" in my_count.keys():
                    if my_count['responseCode'] == 'SUCCESS':
                        etld_lib_functions.logger.info(f"{count_type} count: {my_count['count']}")
                    else:
                        raise Exception(f"{count_type} failed, responseCode: {my_count_service_response},"
                                        f" responseMessage: {my_count_service_response}")
                else:
                    raise Exception(f"{count_type} failed, responseCode: {my_count_service_response},"
                                    f" responseMessage: {my_count_service_response}")
            else:
                raise Exception(f"{count_type} failed, responseCode: {my_count_service_response},"
                                f" responseMessage: {my_count_service_response}")

    except Exception as e:
        etld_lib_functions.logger.error(f"Exception: {e}")
        etld_lib_functions.logger.error(f"{count_type} failed, responseCode: {my_count_service_response},")
        etld_lib_functions.logger.error(f"Potential JSON File corruption or api error detected: {json_file}")
        raise Exception("WAS Application Count Failed.")


def start_msg_was_extract(function_name=""):
    etld_lib_functions.logger.info(f"start {function_name}")


def end_msg_was_extract(function_name=""):
    etld_lib_functions.logger.info(f"end {function_name}")


def main(args=None):
    was_webapp_extract()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='was_04_extract_from_qualys')
    etld_lib_config.main()
    etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main()






#!/usr/bin/env python3
import sys
import timeit
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_config
#from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects

from qualys_etl.etld_pcrs import pcrs_03_extract_controller
from qualys_etl.etld_pcrs import pcrs_05_transform_load_json_to_sqlite
from qualys_etl.etld_pcrs import pcrs_06_distribution
import traceback
import time

global start_time
global stop_time


def pcrs_03_extract_controller_wrapper(
        module_function=pcrs_03_extract_controller, message=""
):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def pcrs_05_transform_load_json_to_sqlite_wrapper(
        module_function=pcrs_05_transform_load_json_to_sqlite, message=""
):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def pcrs_06_distribution_wrapper(
        module_function=pcrs_06_distribution, message=""):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def begin_pcrs_02_workflow_manager():
    global start_time
    start_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"__start__ pcrs_etl_workflow {str(sys.argv)}")


def end_pcrs_workflow_manager():
    global start_time
    global stop_time

    stop_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"runtime for pcrs_etl_workflow in seconds: {stop_time - start_time:,}")
    etld_lib_functions.logger.info(f"__end__ pcrs_etl_workflow {str(sys.argv)}")


def pcrs_etl_workflow():
    try:
        begin_pcrs_02_workflow_manager()
        pcrs_03_extract_controller_wrapper(
            message=f"pcrs_hostids_last_scan_date={etld_lib_config.pcrs_last_scan_date}")
        pcrs_06_distribution_wrapper()
        end_pcrs_workflow_manager()
    except Exception as e:
        time.sleep(10)
        etld_lib_functions.logger.error(f"Error pcrs_02_workflow_manager - pcrs_etl_workflow, "
                                        f"please investigate {sys.argv}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)
    # except KeyboardInterrupt:
    #     time.sleep(10)
    #     etld_lib_functions.logger.error(f"Error pcrs_02_workflow_manager - pcrs_etl_workflow, "
    #                                     f"please investigate {sys.argv}")
    #     etld_lib_functions.logger.error("Caught a KeyboardInterrupt, performing cleanup...")
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)
    # except SystemExit:
    #     time.sleep(10)
    #     etld_lib_functions.logger.error(f"Error pcrs_02_workflow_manager - pcrs_etl_workflow, "
    #                                     f"please investigate {sys.argv}")
    #     etld_lib_functions.logger.error("Caught a SystemExit, performing cleanup...")
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)
    #

def main():
    pcrs_etl_workflow()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='pcrs_02_workflow_manager')
    etld_lib_config.main()
    #etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main()



#!/usr/bin/env python3
from pathlib import Path
import json
import collections
import gzip
#from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_datetime
from qualys_etl.etld_lib import etld_lib_extract_transform_load


def pcrs_extract_policy_list(last_evaluation_date,
                             batch_number_str,
                             qualys_headers_dict,
                             payload_dict,
                             cred_dict,
                             file_info_dict) -> list:

    begin_pcrs_04_extract(message=f"extract of Policy List with last evaluation date: {last_evaluation_date}")
    bearer = cred_dict['bearer']
    #url = f"https://{cred_dict['gateway_fqdn_server']}/pcrs/1.0/posture/policy/list"
    url = f"https://{cred_dict['gateway_fqdn_server']}{etld_lib_config.pcrs_policy_list_api_endpoint}"
    if 'lastEvaluationDate' in payload_dict:
        url = f"{url}?lastEvaluationDate={payload_dict['lastEvaluationDate']}"
    else:
        url = f"{url}?lastEvaluationDate={last_evaluation_date}" # From command line -d option.

    headers = {'X-Requested-With': 'qualysetl', 'Authorization': bearer, 'Content-Type': 'application/json'}
    etld_lib_functions.logger.info(f"api call     - {url}")

    json_file = Path(file_info_dict['next_file_path'])

    cred_dict = etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.pcrs_try_extract_max_count,
        url=url,
        headers=headers,
        payload={},
        http_conn_timeout=etld_lib_config.pcrs_http_conn_timeout,
        chunk_size_calc=etld_lib_config.pcrs_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json',
        request_method="GET"
    )

    policy_list = pcrs_get_policy_list(json_file=json_file)
    end_pcrs_04_extract(message=f"extract of Policy List with last evaluation date: {last_evaluation_date}")
    return policy_list


def pcrs_extract_hostids_list(last_scan_date,
                              last_evaluation_date,
                             batch_number_str,
                             qualys_headers_dict,
                             cred_dict,
                             payload_dict,
                             payload_postureinfo_dict,
                             policy_id,
                             file_info_dict) -> list:

    #begin_pcrs_04_extract(message=f"extract policy hostids with last scan date: {last_scan_date}")
    begin_pcrs_04_extract(message=f"extract policy hostids")            # Use last evaluation date.
    bearer = cred_dict['bearer']
    #url = f"https://{cred_dict['gateway_fqdn_server']}/pcrs/1.0/posture/hostids"
    url = f"https://{cred_dict['gateway_fqdn_server']}{etld_lib_config.pcrs_hostids_api_endpoint}"
    url = f"{url}?policyId={policy_id}"
    if 'lastScanDate' in payload_dict:
        url = f"{url}&lastScanDate={str(payload_dict['lastScanDate'])}"
    elif 'lastScanDate' in payload_postureinfo_dict:
        etld_lib_functions.logger.info("lastScanDate for hostids obtained from payload_postureinfo lastScanDate")
        url = f"{url}&lastScanDate={str(payload_postureinfo_dict['lastScanDate'])}"
    elif 'lastScanDateFrom' in payload_postureinfo_dict:
        etld_lib_functions.logger.info("lastScanDate for hostids obtained from payload_postureinfo lastScanDateFrom")
        url = f"{url}&lastScanDate={str(payload_postureinfo_dict['lastScanDateFrom'])}"
    elif '1970' not in last_evaluation_date: # Default 1970 date
        last_scan_date = etld_lib_datetime.add_or_subtract_hours_from_rfc_3339_datetime(last_evaluation_date,1,'subtract')
        etld_lib_functions.logger.info("lastScanDate derived from lastEvaluationDate minus one hour.")
        url = f"{url}&lastScanDate={last_scan_date}"


    headers = {'X-Requested-With': 'qualysetl', 'Authorization': bearer, 'Content-Type': 'application/json'}
    etld_lib_functions.logger.info(f"api call     - {url}")

    json_file = Path(file_info_dict['next_file_path'])

    cred_dict = etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.pcrs_try_extract_max_count,
        url=url,
        headers=headers,
        payload={},
        http_conn_timeout=etld_lib_config.pcrs_http_conn_timeout,
        chunk_size_calc=etld_lib_config.pcrs_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json',
        request_method="GET"
    )

    #policy_list = pcrs_get_policy_list(json_file=json_file)
    hostids_list = []
    end_pcrs_04_extract(message=f"extract policy hostids")            # Use last evaluation date.
    return hostids_list


def pcrs_extract_postureinfo(last_scan_date,
                             last_evaluation_date,
                              batch_number_str,
                              qualys_headers_dict,
                              cred_dict,
                              policy_id,
                             payload_postureinfo_dict,
                             payload_dict,
                              file_info_dict) -> str:


    begin_pcrs_04_extract(message=f"extract postureinfo batch: {batch_number_str}")
    number_of_hostids = len(payload_postureinfo_dict[0]['hostIds'])
    etld_lib_functions.logger.info(f"postureinfo hostIds submitted: {number_of_hostids:,}")
    bearer = cred_dict['bearer']
    #url = f"https://{cred_dict['gateway_fqdn_server']}/pcrs/1.0/posture/postureInfo" #?evidenceRequired=1&compressionRequired=0
    url = f"https://{cred_dict['gateway_fqdn_server']}{etld_lib_config.pcrs_postureinfo_api_endpoint}" #?evidenceRequired=1&compressionRequired=0
    # Options potentially from etld_config_settings.yaml.
    # lastScanDate, lastEvaluationDate, lastScanDateFrom, lastScanDateTo, statusChangedSince

    url = f"{url}?compressionRequired=1"

    if 'lastScanDate' in payload_dict:
        url = f"{url}&lastScanDate={str(payload_dict['lastScanDate'])}"

    if 'lastEvaluationDate' in payload_dict:
        url = f"{url}&lastEvaluationDate={str(payload_dict['lastEvaluationDate'])}"
    else:
        url = f"{url}&lastEvaluationDate={last_evaluation_date}" # Command Line Selection by Default.

    if 'lastScanDateFrom' in payload_dict and 'lastScanDateTo' in payload_dict:
        url = f"{url}&lastScanDateFrom={str(payload_dict['lastScanDateFrom'])}&lastScanDateTo={str(payload_dict['lastScanDateTo'])}"

    if 'statusChangedSince' in payload_dict:
            url = f"{url}&statusChangedSince={str(payload_dict['statusChangedSince'])}"


    if 'evidenceRequired' in payload_dict:
        url = f"{url}&evidenceRequired={str(payload_dict['evidenceRequired'])}"
    else:
        url = f"{url}&evidenceRequired=1"

    headers = {'X-Requested-With': 'qualysetl', 'Authorization': bearer, 'Content-Type': 'application/json'}
    etld_lib_functions.logger.info(f"api call     - {url}")

    if url.__contains__('compressionRequired=1'):
        compression_method = open
    else:
        compression_method = gzip.open

    json_file = Path(file_info_dict['next_file_path'])
    cred_dict = etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=etld_lib_config.pcrs_try_extract_max_count,
        url=url,
        headers=headers,
        payload=json.dumps(payload_postureinfo_dict),
        http_conn_timeout=etld_lib_config.pcrs_http_conn_timeout,
        chunk_size_calc=etld_lib_config.pcrs_chunk_size_calc,
        output_file=json_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_dict,
        batch_number_formatted=batch_number_str,
        extract_validation_type='json',
        compression_method=compression_method,
        request_method="POST"
    )

    # posture_info_dict = pcrs_get_posture_info_dict(json_file)
    posture_info_chunk_of_characters = pcrs_get_chunk_of_characters_from_file(json_file)
    end_pcrs_04_extract(message=f"extract postureinfo batch: {batch_number_str}, hostIds submitted: {number_of_hostids:,}")
    return posture_info_chunk_of_characters


def pcrs_get_sorted_active_policy_list(policy_list) -> list:

    active_policy_count = 0
    not_active_policy_count = 0
    active_policy_dict = {}
    active_policy_list = []
    sorted_active_policy_list = []
    for policy in policy_list['policyList']:
        if 'status' in policy.keys():
            if policy['status'] == 'active':
                active_policy_count = active_policy_count + 1
                # Create sorted by lastEvaluatedDate policy list
                policy_id = policy['id']
                policy_last_evaluated_date = policy['lastEvaluatedDate']
                sort_key = f"{policy_last_evaluated_date}:{policy_id}"
                active_policy_dict[f"{sort_key}"] = policy
                active_policy_list.append(policy)
            else:
                not_active_policy_count = not_active_policy_count + 1

    sorted_active_policy_dict = collections.OrderedDict(sorted(active_policy_dict.items()))

    for k, v in sorted_active_policy_dict.items():
        sorted_active_policy_list.append(v)

    sorted_active_policy_list.reverse()
    etld_lib_functions.logger.info(f"policyList count active policies: {active_policy_count:,}")
    etld_lib_functions.logger.info(f"policyList count inactive policies: {not_active_policy_count:,}")

    if len(sorted_active_policy_list) == len(active_policy_list):
        pass
    else:
        raise Exception(
            f"Problem sorting active policy list. duplicate key (lastEvaluatedDate:id).")

    return sorted_active_policy_list


def pcrs_get_policy_list(json_file) -> list:
    etld_lib_functions.logger.info(f"pcrs_get_policy_list from file: {json_file}")
    active_policy_list = []
    sorted_active_policy_list = []
    try:
        with etld_lib_config.pcrs_open_file_compression_method(str(json_file), "rt", encoding='utf-8') as read_file:
           policy_list = json.load(read_file)
           if "policyList" in policy_list.keys():
               etld_lib_functions.logger.info(f"policyList file: {json_file}")
               etld_lib_functions.logger.info(f"policyList found, total count of policies: {len(policy_list['policyList'])}")
           else:
               raise Exception(f"Policy List not correctly formed, please review file: {json_file}")

           sorted_active_policy_list = pcrs_get_sorted_active_policy_list(policy_list)

    except Exception as e:
        etld_lib_functions.logger.error(f"Exception: {e}")
        etld_lib_functions.logger.error(f"Potential JSON File corruption or api error detected: {json_file}")
        raise Exception(f"Exception: {e}")

    return sorted_active_policy_list


def pcrs_get_hostids_list(json_file) -> list:
    etld_lib_functions.logger.info(f"pcrs_get_hostids_list from file: {json_file}")
    hostids_list = []
    try:
        with etld_lib_config.pcrs_open_file_compression_method(
                str(json_file), "rt", encoding='utf-8') as read_file:
            hostids_list = json.load(read_file)
    except Exception as e:
        etld_lib_functions.logger.error(f"pcrs_get_hostids_list Exception: {e}")
        etld_lib_functions.logger.error(f"pcrs_get_hostids_list Potential JSON File corruption or api error detected: {json_file}")
        exit(1)
    return hostids_list


# def pcrs_get_posture_info_dict(json_file) -> dict:
#     etld_lib_functions.logger.info(f"pcrs_get_posture_info_dict from file: {json_file}")
#     posture_info = {}
#     try:
#         with etld_lib_config.pcrs_open_file_compression_method(
#                 str(json_file), "rt", encoding='utf-8') as read_file:
#             posture_info = json.load(read_file)
#     except Exception as e:
#         etld_lib_functions.logger.error(f"pcrs_get_hostids_list Exception: {e}")
#         etld_lib_functions.logger.error(f"pcrs_get_hostids_list Potential JSON File corruption or api error detected: {json_file}")
#         exit(1)
#     return posture_info


def pcrs_get_chunk_of_characters_from_file(file_name, number_of_characters=100) -> str:
    etld_lib_functions.logger.info(f"pcrs_get_chunk_of_characters_from_file: {file_name}")
    try:
        with etld_lib_config.pcrs_open_file_compression_method(
                str(file_name), "rt", encoding='utf-8') as read_file:
            get_chunk = read_file.read(number_of_characters)
    except Exception as e:
        etld_lib_functions.logger.error(f"pcrs_get_chunk_of_characters_from_file Exception: {e}")
        etld_lib_functions.logger.error(
            f"Potential File corruption or api error detected: {file_name}")
        exit(1)
    return get_chunk

# def pcrs_get_policy_list(json_file) -> list:
#     try:
#         with etld_lib_config.pcrs_open_file_compression_method(
#                 str(json_file), "rt", encoding='utf-8') as read_file:
#             policy_list = json.load(read_file)
#
#             if "policyList" in policy_list.keys():
#                 etld_lib_functions.logger.info(f"pcrs get policyList from file: {json_file}")
#             else:
#                 raise Exception(f"pcrs policy list not correctly formed, please review file: {json_file}")
#             return policy_list['policyList']
#
#     except Exception as e:
#         etld_lib_functions.logger.error(f"pcrs_get_policy_list Exception: {e}")
#         etld_lib_functions.logger.error(f"pcrs_get_policy_list Potential JSON File corruption or api error detected: {json_file}")
#         exit(1)


def begin_pcrs_04_extract(message=""):
    etld_lib_functions.logger.info(f"start {message}")


def end_pcrs_04_extract(message=""):
    etld_lib_functions.logger.info(f"end   {message}")


def main(args=None):
    etld_lib_authentication_objects.qualys_authentication_obj.get_current_bearer_token()
    credentials_dict = etld_lib_authentication_objects.qualys_authentication_obj.get_credentials_dict()
    # TODO FOR TESTING, CREATE TEST BATCH OPTIONS HERE.
    # pcrs_extract(cred_dict=credentials_dict)


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='pcrs_04_extract_from_qualys')
    etld_lib_config.main()
    #etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main()






#!/usr/bin/env python3
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_test_system


def main():
    etld_lib_test_system.main()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='test_system_03_controller')
    etld_lib_config.main()
    etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main()



#!/usr/bin/env python3
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_spawn_etl
from qualys_etl.etld_data_flow import df_02_workflow_manager


def main():
    prefix_name = etld_lib_config.df_workflow_prefix_name # Set in qetl_manage_user
    etld_lib_spawn_etl.etl_main(etl_workflow_prefix=prefix_name)

if __name__ == '__main__':
    main()



import os
import stat

def set_mode(filename, mode_str):
    mode = 0

    for i, permissions in enumerate(reversed(mode_str.split(','))):
        if 'u' in permissions:
            if '+' in permissions:
                if 'r' in permissions:
                    mode |= stat.S_IRUSR
                if 'w' in permissions:
                    mode |= stat.S_IWUSR
                if 'x' in permissions:
                    mode |= stat.S_IXUSR
            elif '-' in permissions:
                if 'r' in permissions:
                    mode &= ~stat.S_IRUSR
                if 'w' in permissions:
                    mode &= ~stat.S_IWUSR
                if 'x' in permissions:
                    mode &= ~stat.S_IXUSR
        elif 'g' in permissions:
            if '+' in permissions:
                if 'r' in permissions:
                    mode |= stat.S_IRGRP
                if 'w' in permissions:
                    mode |= stat.S_IWGRP
                if 'x' in permissions:
                    mode |= stat.S_IXGRP
            elif '-' in permissions:
                if 'r' in permissions:
                    mode &= ~stat.S_IRGRP
                if 'w' in permissions:
                    mode &= ~stat.S_IWGRP
                if 'x' in permissions:
                    mode &= ~stat.S_IXGRP
        elif 'o' in permissions:
            if '+' in permissions:
                if 'r' in permissions:
                    mode |= stat.S_IROTH
                if 'w' in permissions:
                    mode |= stat.S_IWOTH
                if 'x' in permissions:
                    mode |= stat.S_IXOTH
            elif '-' in permissions:
                if 'r' in permissions:
                    mode &= ~stat.S_IROTH
                if 'w' in permissions:
                    mode &= ~stat.S_IWOTH
                if 'x' in permissions:
                    mode &= ~stat.S_IXOTH

    os.chmod(filename, mode)


from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()
here_qualys_etl = pathlib.Path(here, "qualys_etl")

# Get the long description from the README file
long_description = (here_qualys_etl / 'README.md').read_text(encoding='utf-8')


setup(
    name='qualysetl',
    version='0.9.1',
    packages=find_packages(include=[
        'qualys_etl',
        'qualys_etl.etld_lib',
        'qualys_etl.etld_host_list',
        'qualys_etl.etld_templates',
        'qualys_etl.etld_knowledgebase',
        'qualys_etl.etld_host_list_detection',
        'qualys_etl.etld_asset_inventory',
        'qualys_etl.etld_pcrs',
        'qualys_etl.*',
        'qualys_etl.etld_lib.*',
        'qualys_etl.etld_host_list.*',
        'qualys_etl.etld_templates.*',
        'qualys_etl.etld_knowledgebase.*',
        'qualys_etl.etld_host_list_detection.*',
        'qualys_etl.etld_asset_inventory.*',
        'qualys_etl.etld_was.*',
        'qualys_etl.etld_pcrs.*',
    ]),
    scripts=['bin/qetl_setup_python_venv'],
    url='https://pypi.org/project/qualysetl/',
    project_urls={
        'Documentation': 'https://dg-cafe.github.io/qualysetl/',
        'Qualys Video Series': 'https://blog.qualys.com/tag/api-best-practices-series',
    },
    keywords='qualys, etl, qualys.com, david gregory, qualysetl, qualysapi',
    license='Apache',
    author='David Gregory',
    author_email='dgregory@qualys.com, dave@davidgregory.com',
    description='Qualys API Best Practices Series - ETL Blueprint Example Code within Python Virtual Environment',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3 :: Only',
        'Operating System :: POSIX :: Linux',
    ],
    python_requires='>=3.8.5',
    package_data={'': ['*.yaml', '.*.yaml', '*.sh', '*.md']},
)



#!/usr/bin/env python3
import multiprocessing
import sys
import tempfile
import os
import time
import signal
from pathlib import Path
import traceback

class QueueWriter:
    def __init__(self, queue):
        self.queue = queue

    def write(self, message):
        if message != '\n':
            self.queue.put(message)

    def flush(self):
        self.queue.put('FLUSH')

def listener(queue, log_file_path):
    with open(log_file_path, 'a', newline='', encoding='utf-8') as log_file:
        while True:
            message = queue.get()
            if message == 'STOP':
                break
            if message == 'FLUSH':
                log_file.flush()
            elif message.startswith('SIGNAL:'):
                sys.stderr.write(f"{message}\n")
                sys.stderr.flush()
            else:
                log_file.write(message)
                log_file.flush()

def signal_handler(signum, frame, queue):
    # Convert the signal number to a human-readable string, e.g., 'SIGINT'
    signal_name = signal.Signals(signum).name
    queue.put(f'SIGNAL: Received {signal_name}')

def setup_signal_handlers(queue):
    # Register the signal handler for SIGINT and SIGTERM
    for sig in (signal.SIGINT, signal.SIGTERM):
        signal.signal(sig, lambda signum, frame: signal_handler(signum, frame, queue))


def main():
    log_queue = multiprocessing.Queue()
    with tempfile.NamedTemporaryFile(delete=False) as temp_log_file:
        log_file_path = temp_log_file.name

    print(f"LOGFILE: {log_file_path}")
    listener_proc = multiprocessing.Process(target=listener, args=(log_queue, log_file_path))
    listener_proc.start()

    setup_signal_handlers(log_queue)
    sys.stdout = QueueWriter(log_queue)
    sys.stderr = QueueWriter(log_queue)

    try:
        while True:
            user_input = input("Enter something (or Ctrl+C to exit): ") + "\n"
            print(f"LOGGING: {user_input}")
    except KeyboardInterrupt:
        print("Caught a KeyboardInterrupt, performing cleanup...", file=sys.__stderr__)
    except Exception as e:
        print(f"Caught Exception {e}...", file=sys.__stderr__)
        traceback.print_exc(file=sys.__stderr__)
    finally:
        print("Performing cleanup...", file=sys.__stderr__)
        log_queue.put('STOP')
        listener_proc.join()
        exit


if __name__ == "__main__":
    main()